"""
Market Price Checks cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot
"""

# Standard Library
import locale
import logging

# Third Party
import requests
from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

# Alliance Auth (External Libs)
from eveuniverse.models import EveEntity

logger = logging.getLogger(__name__)


class PriceCheck(commands.Cog):
    """
    Price checks on Jita, Amarr, Rens, Hek, Dodixie and Perimeter markets
    """

    imageserver_url = "https://images.evetech.net"

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def price(self, ctx):
        """
        Check an item price on all major market hubs

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        markets = [
            {"name": "Jita", "system_id": 30000142},
            {"name": "Amarr", "system_id": 30002187},
            {"name": "Rens", "system_id": 60004588},
            {"name": "Hek", "system_id": 60005686},
            {"name": "Dodixie", "system_id": 30002659},
        ]

        await ctx.trigger_typing()

        item_name = ctx.message.content[7:]

        await self.price_check(ctx=ctx, markets=markets, item_name=item_name)

    @commands.command(pass_context=True)
    async def jita(self, ctx):
        """
        Check an item price on Jita market

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        markets = [{"name": "Jita", "system_id": 30000142}]

        await ctx.trigger_typing()

        item_name = ctx.message.content[6:]

        await self.price_check(ctx=ctx, markets=markets, item_name=item_name)

    @commands.command(pass_context=True)
    async def amarr(self, ctx):
        """
        Check an item price on Amarr market

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        markets = [{"name": "Amarr", "system_id": 60008494}]

        await ctx.trigger_typing()

        item_name = ctx.message.content[7:]

        await self.price_check(ctx=ctx, markets=markets, item_name=item_name)

    async def price_check(self, ctx, markets, item_name: str = None):
        """
        Do the price checks and post to Discord

        :param ctx:
        :type ctx:
        :param markets:
        :type markets:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        has_thumbnail = False

        await ctx.trigger_typing()

        if item_name != "":
            try:
                eve_type = (
                    EveEntity.objects.fetch_by_names_esi([item_name])
                    .filter(category=EveEntity.CATEGORY_INVENTORY_TYPE)
                    .values_list("id", flat=True)
                )

                eve_type_id = str(eve_type[0])
            except (EveEntity.DoesNotExist, IndexError):
                embed = Embed(
                    title=f"Price Lookup for {item_name}",
                    color=Color.orange(),
                )

                embed.add_field(
                    name="Error",
                    value=(
                        f"{item_name} could not be found. "
                        "Are you sure you spelled it correctly?"
                    ),
                    inline=False,
                )
            else:
                embed = Embed(
                    title=f"Price Lookup for {item_name}",
                    color=Color.green(),
                )

                for market in markets:
                    embed.add_field(
                        name=market["name"],
                        value=f'Prices for {item_name} on the {market["name"]} Market.',
                        inline=False,
                    )

                    system_id = market["system_id"]
                    url = "https://market.fuzzwork.co.uk/aggregates/"
                    url_params = {"system": system_id, "types": eve_type_id}
                    market_data = requests.get(url=url, params=url_params, timeout=2.50)

                    if market_data.status_code == 200:
                        market_json = market_data.json()

                        sell_min = market_json[eve_type_id]["sell"]["min"]
                        sell_order_count = market_json[eve_type_id]["sell"][
                            "orderCount"
                        ]
                        buy_max = market_json[eve_type_id]["buy"]["max"]
                        buy_order_count = market_json[eve_type_id]["buy"]["orderCount"]
                        thumbnail_url = (
                            f"{self.imageserver_url}/types/{eve_type_id}/icon?size=64"
                        )

                        if has_thumbnail is False:
                            embed.set_thumbnail(url=thumbnail_url)
                            has_thumbnail = True

                        locale.setlocale(category=locale.LC_ALL, locale="")

                        # Sell order price
                        market_min_sell_order_price = locale.format_string(
                            f="%.2f", val=float(sell_min), grouping=True
                        )

                        if sell_order_count == 0:
                            market_min_sell_order_price = "No sell orders found"

                        embed.add_field(
                            name=f"Sell Order Price ({sell_order_count} Orders)",
                            value=f"{market_min_sell_order_price} ISK",
                            inline=True,
                        )

                        # Buy order price
                        market_max_buy_order_price = locale.format_string(
                            f="%.2f", val=float(buy_max), grouping=True
                        )

                        if buy_order_count == 0:
                            market_max_buy_order_price = "No buy orders found"

                        embed.add_field(
                            name=f"Buy Order Price ({buy_order_count} Orders)",
                            value=f"{market_max_buy_order_price} ISK",
                            inline=True,
                        )
                    else:
                        embed.add_field(
                            name="API Error",
                            value=(
                                "Could not not fetch the price "
                                f'for the {market["name"]} market.'
                            ),
                            inline=False,
                        )
        else:
            embed = Embed(
                title="Price Lookup",
                color=Color.red(),
            )

            embed.add_field(
                name="Error",
                value=(
                    "You forget to enter an item you want to lookup the price for ..."
                ),
                inline=False,
            )

        return await ctx.send(embed=embed)


def setup(bot):
    """
    Set up the cogg

    :param bot:
    :type bot:
    :return:
    :rtype:
    """

    bot.add_cog(PriceCheck(bot))



"""
"About" cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot

Since we don't want to have it branded for "The Initiative" we have to build our own
"""

# Standard Library
import logging

# Third Party
import pendulum
from aadiscordbot.app_settings import get_site_url
from aadiscordbot.cogs.utils.decorators import sender_is_admin
from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

# Django
from django.conf import settings

# Alliance Auth
from allianceauth.eveonline.evelinks.eveimageserver import (
    alliance_logo_url,
    corporation_logo_url,
)

logger = logging.getLogger(name=__name__)


class About(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def about(self, ctx):
        """
        All about the bot

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        await ctx.trigger_typing()

        auth_url = get_site_url()
        embed = Embed(title="TN-NT Discord Services")

        try:
            if settings.TNNT_TEMPLATE_ENTITY_ID == 1:
                aa_icon = f"{auth_url}/static/allianceauth/icons/allianceauth.png"
                embed.set_thumbnail(url=aa_icon)
            else:
                if settings.TNNT_TEMPLATE_ENTITY_TYPE == "alliance":
                    embed.set_thumbnail(
                        url=alliance_logo_url(
                            alliance_id=settings.TNNT_TEMPLATE_ENTITY_ID, size=256
                        )
                    )
                elif settings.TNNT_TEMPLATE_ENTITY_TYPE == "corporation":
                    embed.set_thumbnail(
                        url=corporation_logo_url(
                            corporation_id=settings.TNNT_TEMPLATE_ENTITY_ID, size=256
                        )
                    )
        except AttributeError:
            pass

        embed.colour = Color.green()

        embed.description = (
            "This is a multi-functional discord bot tailored "
            "specifically for Terra Nanotech.\n\nType `!help` for more information. "
        )

        embed.add_field(name="Auth Link", value=auth_url, inline=False)

        embed.set_footer(
            text="Developed by Aaron Kable, forked for Terra Nanotech by Rounon Dax"
        )

        return await ctx.send(embed=embed)

    @commands.command(hidden=True)
    @sender_is_admin()
    async def uptime(self, ctx):
        """
        Returns the uptime

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        await ctx.send(
            pendulum.now(tz="UTC").diff_for_humans(
                self.bot.currentuptime, absolute=True
            )
        )


def setup(bot):
    """
    Set up the cog

    :param bot:
    :type bot:
    :return:
    :rtype:
    """

    bot.add_cog(About(bot))



"""
Application init
"""

# Standard Library
from importlib import metadata

__version__ = metadata.version(distribution_name="tnnt-discordbot-cogs")
__title__ = "TN-NT Discordbot Cogs"

del metadata



"""
All about the timers …
"""

# Standard Library
import datetime
import logging

# Third Party
from aadiscordbot.app_settings import ADMIN_DISCORD_BOT_CHANNELS
from aadiscordbot.cogs.utils.decorators import message_in_channels
from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands
from structuretimers.models import Timer

# Django
from django.apps import apps
from django.utils import timezone

# Alliance Auth
from allianceauth.eveonline.templatetags.evelinks import dotlan_solar_system_url

# Alliance Auth (External Libs)
from app_utils.urls import reverse_absolute

logger = logging.getLogger(__name__)


def structuretimers_active():
    """
    Check if "structuretimers" is installed
    :return:
    :rtype:
    """

    return apps.is_installed("structuretimers")


def timezones_active():
    """
    Check if "timezones" is installed
    :return:
    :rtype:
    """

    return apps.is_installed("timezones")


def add_empty_line(embed: Embed) -> None:
    """
    Adding an empty line to the embed

    :param embed:
    :type embed:
    :return:
    :rtype:
    """

    embed.add_field(
        name="\u200b",
        value="\u200b",
        inline=False,
    )


class Timers(commands.Cog):
    """
    TimerBoard Stuffs!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    @message_in_channels(ADMIN_DISCORD_BOT_CHANNELS)
    async def timer(self, ctx):
        """
        Gets the Next Timer!

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        embed = Embed(title="Next Timer")

        next_timer = Timer.objects.filter(
            is_opsec=False,
            visibility=Timer.VISIBILITY_UNRESTRICTED,
            date__gte=datetime.datetime.utcnow().replace(tzinfo=timezone.utc),
        ).first()

        if next_timer is None:
            embed.description = "No upcoming timer"
        else:
            if next_timer.objective == Timer.OBJECTIVE_FRIENDLY:
                embed.colour = Color.blue()
            elif next_timer.objective == Timer.OBJECTIVE_HOSTILE:
                embed.colour = Color.red()
            elif next_timer.objective == Timer.OBJECTIVE_NEUTRAL:
                embed.colour = Color.light_gray()
            elif next_timer.objective == Timer.OBJECTIVE_UNDEFINED:
                embed.colour = Color.dark_gray()
            else:
                embed.colour = Color.from_rgb(r=255, g=255, b=255)

            user_has_profile = True

            if next_timer.user is not None:
                creator_name = next_timer.user.username

                try:
                    creator_profile = next_timer.user.profile
                except Exception:
                    user_has_profile = False

                if user_has_profile is True:
                    if creator_profile.main_character is not None:
                        creator_name = creator_profile.main_character.character_name

                embed.set_footer(text=f"Timer added by {creator_name}")

            embed.add_field(name="Structure:", value=next_timer.structure_type.name)

            solar_system_name = next_timer.eve_solar_system.name
            solar_system_link = dotlan_solar_system_url(next_timer.eve_solar_system)
            solar_system_md_link = f"[{solar_system_name}]({solar_system_link})"
            location_details = next_timer.location_details

            embed.add_field(
                name="Location:", value=f"{solar_system_md_link}\n{location_details}"
            )

            add_empty_line(embed=embed)

            timer_timestamp = str(int(next_timer.date.timestamp()))

            if timezones_active():
                timezones_url = reverse_absolute(
                    viewname="timezones:index",
                    args=[timer_timestamp],
                )
                timezones_md_link = f"[TZ Conversion]({timezones_url})"
                timer_eve_time = next_timer.date.strftime("%Y-%m-%d %H:%M")

                embed.add_field(
                    name="Eve Time:",
                    value=f"{timer_eve_time} ({timezones_md_link})",
                    inline=True,
                )
            else:
                embed.add_field(
                    name="Eve Time:",
                    value=next_timer.eve_time.strftime("%Y-%m-%d %H:%M"),
                    inline=True,
                )

            embed.add_field(
                name="Local Time:",
                value=f"<t:{timer_timestamp}:F>",
                inline=True,
            )

        return await ctx.send(embed=embed)


def setup(bot):
    """
    Set up the cog

    :param bot:
    :type bot:
    :return:
    :rtype:
    """

    if structuretimers_active():
        bot.add_cog(Timers(bot))
    else:
        logger.debug("Timerboard not installed")
